using System;
using System.Windows.Forms;

using KOEI.WP8_2014;
using KOEI.WP8_2014.Horse;
using KOEI.WP8_2014.Datastruct;

/// <summary>
/// 指定種牡馬の血統支配率を表示するスクリプト
/// </summary>
class WP8_2014ULV_Extension {

	public static HSireData CommandMain( WP8 wp, UInt32 horse_num, HSireData data ) {
		var disp = new DispSyuSihai( wp, horse_num );
		return data;
	}
}

class DispSyuSihai
{
	private WP8 wp_;

	private uint JSHyo = 0;			// 調査種牡馬とその子孫種牡馬の評価額合計
	private uint USHyo = 0;
	private uint ESHyo = 0;

	private uint JSToHyo = 0;		// 全種牡馬の評価額合計
	private uint USToHyo = 0;
	private uint ESToHyo = 0;

	private uint JSNum = 0;			// 調査種牡馬とその子孫種牡馬の数
	private uint USNum = 0;
	private uint ESNum = 0;

	private uint JSToNum = 0;		// 全種牡馬の数
	private uint USToNum = 0;
	private uint ESToNum = 0;

	private uint JHNum = 0;			// 調査種牡馬を父系に持つ繁殖牝馬の数
	private uint UHNum = 0;
	private uint EHNum = 0;

	private uint JHToNum = 0;		// 全繁殖牝馬の数
	private uint UHToNum = 0;
	private uint EHToNum = 0;

	private uint JKNum = 0;			// 調査種牡馬を父系に持つ競走馬の数
	private uint UKNum = 0;
	private uint EKNum = 0;

	private uint JKToNum = 0;		// 全競走馬の数
	private uint UKToNum = 0;
	private uint EKToNum = 0;

	public DispSyuSihai( WP8 wp, UInt32 horse_num )
	{
		this.wp_ = wp;

		GetSyuData( horse_num );
		GetHanData( horse_num );
		GetKyoData( horse_num );

		var sire_data  = new HSireData();
		var blood_data = new HBloodData();

		wp.HSireTable.GetData( horse_num, ref sire_data );
		wp.HBloodTable.GetData( sire_data.blood_num, ref blood_data );

		string msg;

/***
		msg = String.Format( "日本全:{0}　米国全:{1}　欧州全:{2}\n日本該:{3}　米国該:{4}　欧州該:{5}",
							 JSToHyo, USToHyo, ESToHyo, JSHyo, USHyo, ESHyo );
		MessageBox.Show( msg, String.Format( "{0} - 種付け料関連", get_name_by_data( ref blood_data ) ) );

		msg = String.Format( "日本全:{0}　米国全:{1}　欧州全:{2}\n日本該:{3}　米国該:{4}　欧州該:{5}",
							 JSToNum, USToNum, ESToNum, JSNum, USNum, ESNum );
		MessageBox.Show( msg, String.Format( "{0} - 種牡馬数関連", get_name_by_data( ref blood_data ) ) );

		msg = String.Format( "日本全:{0}　米国全:{1}　欧州全:{2}\n日本該:{3}　米国該:{4}　欧州該:{5}",
							 JHToNum, UHToNum, EHToNum, JHNum, UHNum, EHNum );
		MessageBox.Show( msg, String.Format( "{0} - 繁殖牝馬数関連", get_name_by_data( ref blood_data ) ) );

		msg = String.Format( "日本全:{0}　米国全:{1}　欧州全:{2}\n日本該:{3}　米国該:{4}　欧州該:{5}",
							 JKToNum, UKToNum, EKToNum, JKNum, UKNum, EKNum );
		MessageBox.Show( msg, String.Format( "{0} - 競走馬数関連", get_name_by_data( ref blood_data ) ) );
***/

		double JSHyoPer = (double)JSHyo*100/(double)JSToHyo;
		double USHyoPer = (double)USHyo*100/(double)USToHyo;
		double ESHyoPer = (double)ESHyo*100/(double)ESToHyo;
//		double WSHyoPer = (double)(JSHyo+USHyo+ESHyo)*100/(double)(JSToHyo+USToHyo+ESToHyo);

		double JSNumPer = (double)JSNum*100/(double)JSToNum;
		double USNumPer = (double)USNum*100/(double)USToNum;
		double ESNumPer = (double)ESNum*100/(double)ESToNum;
//		double WSNumPer = (double)(JSNum+USNum+ESNum)*100/(double)(JSToNum+USToNum+ESToNum);

		double JHNumPer = (double)JHNum*100/(double)JHToNum;
		double UHNumPer = (double)UHNum*100/(double)UHToNum;
		double EHNumPer = (double)EHNum*100/(double)EHToNum;
//		double WHNumPer = (double)(JHNum+UHNum+EHNum)*100/(double)(JHToNum+UHToNum+EHToNum);

		double JKNumPer = (double)JKNum*100/(double)JKToNum;
		double UKNumPer = (double)UKNum*100/(double)UKToNum;
		double EKNumPer = (double)EKNum*100/(double)EKToNum;
//		double WKNumPer = (double)(JKNum+UKNum+EKNum)*100/(double)(JKToNum+UKToNum+EKToNum);

		double JSihai = (JSHyoPer+JSNumPer+JHNumPer+JKNumPer)/4;
		double USihai = (USHyoPer+USNumPer+UHNumPer+UKNumPer)/4;
		double ESihai = (ESHyoPer+ESNumPer+EHNumPer+EKNumPer)/4;
//		double WSihai = (WSHyoPer+WSNumPer+WHNumPer+WKNumPer)/4;
		double WSihai = (JSihai+USihai+ESihai)/3;

		msg = String.Format( "日本:{0:f}　米国:{1:f}　欧州:{2:f}　世界:{3:f}",
							 JSihai, USihai, ESihai, WSihai );
		MessageBox.Show( msg, String.Format( "{0}の血統支配率", get_name_by_data( ref blood_data ) ) );
	}

	// 種牡馬の評価額合計と種牡馬数を取得
	private void GetSyuData( UInt32 horse_num )
	{
		var wp = this.wp_;
		var data = new HSireData();
		var abl = new HAblData();

		// 調査種牡馬の血統番号を取得
		wp.HSireTable.GetData( horse_num, ref data );
		var tyousa_blood_num = data.blood_num;

		// 全種牡馬を調査し、評価額合計などを取得
		for ( int i=0; i<wp.HSireTable.RecordCount; i++ )
		{
			wp.HSireTable.GetData( (uint)i, ref data );

			// 無効データは飛ばす
			if( data.abl_num == wp.Config.NullAbilityNumber )
				continue;

			wp.HAblTable.GetData( data.abl_num, ref abl );

			// 現役種牡馬だった場合は評価額合計と種牡馬数をカウント
			if ( data.intai == 0 )
			{
				switch( this.GetArea( abl.bokuzyou ) )
				{
					case 0:
						JSToHyo += data.tanetsuke;
						JSToNum ++;
						break;
					case 1:
						USToHyo += data.tanetsuke;
						USToNum ++;
						break;
					case 2:
						ESToHyo += data.tanetsuke;
						ESToNum ++;
						break;
				}
			}

			// 調査種牡馬自身またはその子孫かどうかを判定
			if ( IsFukei( data.blood_num, tyousa_blood_num ) )
			{
				// 現役種牡馬だった場合は評価額合計と種牡馬数をカウント
				if ( data.intai == 0 )
				{
					switch( this.GetArea( abl.bokuzyou ) )
					{
						case 0:
							JSHyo += data.tanetsuke;
							JSNum ++;
							break;
						case 1:
							USHyo += data.tanetsuke;
							USNum ++;
							break;
						case 2:
							ESHyo += data.tanetsuke;
							ESNum ++;
							break;
					}
				}
			}
		}
	}

	// 繁殖牝馬数を取得
	private void GetHanData( UInt32 horse_num )
	{
		var wp = this.wp_;
		var data = new HDamData();
		var sire = new HSireData();
		var abl = new HAblData();
		var blood = new HBloodData();

		// 調査種牡馬の血統番号を取得
		wp.HSireTable.GetData( horse_num, ref sire );
		var tyousa_blood_num = sire.blood_num;

		// 全繁殖牝馬を調査し、繁殖牝馬数を取得
		for ( int i=0; i<wp.HDamTable.RecordCount; i++ )
		{
			wp.HDamTable.GetData( (uint)i, ref data );

			// 無効データは飛ばす
			if( data.abl_num == wp.Config.NullAbilityNumber )
				continue;

			wp.HAblTable.GetData( data.abl_num, ref abl );

			// 現役繁殖牝馬だった場合は繁殖牝馬数をカウント
			if ( data.intai == 0 )
			{
				switch( this.GetArea( abl.bokuzyou ) )
				{
					case 0:
						JHToNum ++;
						break;
					case 1:
						UHToNum ++;
						break;
					case 2:
						EHToNum ++;
						break;
				}
			}

			// 繁殖牝馬の父系に調査種牡馬が存在するかどうかを判定
			wp.HBloodTable.GetData( data.blood_num, ref blood );
			if ( IsFukei( blood.father_num, tyousa_blood_num ) )
			{
				// 現役繁殖牝馬だった場合は繁殖牝馬数をカウント
				if ( data.intai == 0 )
				{
					switch( this.GetArea( abl.bokuzyou ) )
					{
						case 0:
							JHNum ++;
							break;
						case 1:
							UHNum ++;
							break;
						case 2:
							EHNum ++;
							break;
					}
				}
			}
		}
	}

	// 競走馬数を取得
	private void GetKyoData( UInt32 horse_num )
	{
		var wp = this.wp_;
		var data = new HRaceData();
		var sire = new HSireData();
		var abl = new HAblData();

		// 調査種牡馬の血統番号を取得
		wp.HSireTable.GetData( horse_num, ref sire );
		var tyousa_blood_num = sire.blood_num;

		// 全競走馬を調査し、競走馬数を取得
		for ( int i=0; i<wp.HRaceTable.RecordCount; i++ )
		{
			wp.HRaceTable.GetData( (uint)i, ref data );

			// 無効データは飛ばす
			if( data.abl_num == wp.Config.NullAbilityNumber )
				continue;

			wp.HAblTable.GetData( data.abl_num, ref abl );

			// 競走馬数をカウント
			switch( this.GetArea( abl.bokuzyou ) )
			{
				case 0:
					JKToNum ++;
					break;
				case 1:
					UKToNum ++;
					break;
				case 2:
					EKToNum ++;
					break;
			}

			// 競走馬の父系に調査種牡馬が存在するかどうかを判定
			wp.HSireTable.GetData( data.father_num, ref sire );
			if ( IsFukei( sire.blood_num, tyousa_blood_num ) )
			{
				// 競争馬数をカウント
				switch( this.GetArea( abl.bokuzyou ) )
				{
					case 0:
						JKNum ++;
						break;
					case 1:
						UKNum ++;
						break;
					case 2:
						EKNum ++;
						break;
				}
			}
		}
	}

	// 血統番号Aの父系に血統番号Bの馬が存在するか（血統番号AとBが同一だった場合も真とする）
	private bool IsFukei( uint blood_num_A, uint blood_num_B )
	{
		// 自分自身だった場合
		if ( blood_num_A == blood_num_B )
			return true;

		var wp = this.wp_;
		var blood = new HBloodData();
		wp.HBloodTable.GetData( blood_num_A, ref blood );

		// 遡るのは5代までとする
		for( int i=0; i<5; i++ )
		{
			// 父馬のデータがない場合は抜ける
			if ( blood.father_num == wp.Config.NullBloodNumber )
				break;

			// 父馬が調査馬だった
			if ( blood.father_num == blood_num_B )
				return true;

			// 上に該当しなかった場合は次の代へ遡る
			var fnum = blood.father_num;
			wp.HBloodTable.GetData( fnum, ref blood );
		}

		return false;
	}

	// 繋養牧場から地域を取得
	private int GetArea( UInt32 bokuzyou_num )
	{
		if( bokuzyou_num < 220 && bokuzyou_num != 32 && bokuzyou_num != 33 ) {
			return 0; // 日本
		} else if( bokuzyou_num < 239 && bokuzyou_num != 33 ) {
			return 1; // 米国
		}
		return 2; // 欧州
	}

	private string get_name_by_data( ref HBloodData data )
	{
		var wp = this.wp_;
		var name = "";

		var obj = wp.HNameTable[ (int)data.name_left ];
		name = obj != null ? obj.Kana : String.Format ("(不明{0})", data.name_left );

		if( data.name_right != wp.Config.NullNameNumber ) {
			obj = wp.HNameTable[ (int)data.name_right ];
			name += obj != null ? obj.Kana : String.Format ("(不明{0})", data.name_right );
		}
		return name;
	}
}
